/*
 * Copyright (c) 1992, 1996, 1998, 2002, 2009
 *	Craig Leres
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code
 * distributions retain the above copyright notice and this paragraph
 * in its entirety, and (2) distributions including binary code include
 * the above copyright notice and this paragraph in its entirety in
 * the documentation or other materials provided with the distribution
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static const char copyright[] =
    "@(#) Copyright (c) 1992, 1996, 1998, 2002, 2009\nCraig Leres\n";
static const char rcsid[] =
    "@(#) $Id: captive.c 22 2009-12-13 00:14:57Z leres $ (XSE)";
#endif

#include <sys/param.h>
#ifdef HAVE_JAIL
#include <sys/jail.h>
#endif

#include <errno.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

/* Forwards */
int main(int, char **);

int
main(int argc, char **argv)
{
	char *cp;
	uid_t uid, euid;
	struct passwd *pwd;
	char *csh = "/bin/csh";
	char *name = "-csh";
	char root[128];
#ifdef HAVE_JAIL
	char hostname[MAXHOSTNAMELEN];
	struct jail jailinfo;
#endif

	openlog("captive", 0, LOG_AUTH);
	uid = getuid();
	if (uid == 0) {
		syslog(LOG_ERR, "uid == 0");
		exit(1);
	}
	euid = geteuid();
	if (euid != 0) {
		syslog(LOG_ERR, "euid != 0 (%d)", euid);
		exit(1);
	}

	cp = "captive";
	pwd = getpwnam(cp);
	if (pwd == NULL) {
		syslog(LOG_ERR, "Can't find %s account", cp);
		exit(1);
	}
	(void)strcpy(root, pwd->pw_dir);

	pwd = getpwuid(uid);
	if (pwd == NULL) {
		syslog(LOG_ERR, "No such user with uid %d", uid);
		exit(1);
	}

	syslog(LOG_DEBUG, "%s", pwd->pw_name);

#ifdef HAVE_JAIL
	memset(&jailinfo, 0, sizeof(jailinfo));
#ifdef JAIL_API_VERSION
	jailinfo.version = JAIL_API_VERSION;
#endif
	jailinfo.path = root;
	if (gethostname(hostname, sizeof(hostname)) < 0) {
		syslog(LOG_ERR, "gethostname: %m");
		exit(1);
	}
	jailinfo.hostname = hostname;
	if (jail(&jailinfo) < 0) {
		syslog(LOG_ERR, "jail %s: %m", root);
		exit(1);
	}
#else
	if (chroot(root) < 0) {
		syslog(LOG_ERR, "chroot %s: %m", root);
		exit(1);
	}
	if (chdir(pwd->pw_dir) < 0) {
		syslog(LOG_ERR, "chdir %s: %m", pwd->pw_dir);
		exit(1);
	}
#endif
	if (setgid(pwd->pw_gid) < 0) {
		syslog(LOG_ERR, "setgid %d: %m", pwd->pw_gid);
		exit(1);
	}
	if (setuid(pwd->pw_uid) < 0) {
		syslog(LOG_ERR, "setuid %d: %m", pwd->pw_uid);
		exit(1);
	}

	/* Remove leading dash if not interactive (rshd doesn't set TERM) */
	if (getenv("TERM") == NULL)
		++name;
	argv[0] = name;
	execv(csh, argv);

	printf("execv failed: %s\n", strerror(errno));
	syslog(LOG_ERR, "exec \"%s\": %m", csh);
	exit(1);
}
