#!/bin/sh
# @(#) $Id: updatecaptive.sh 23 2009-12-13 00:16:49Z leres $ (LBL)
#
# updatecaptive.sh - update captive chroot bins and libs
#
# sends an update report to stdout if changes were made
#

umask 022

prog=`basename $0`
t1=/tmp/${prog}.1.$$

trap 'rm -f ${t1}; exit 1' 1 2 3 15

PATH="${PATH}:/usr/local/bin"
export PATH

usage() {
	echo "usage: ${prog} -n" 1>&2
	exit 1
}

args=`getopt n $*`
if [ $? -ne 0 ]; then
	usage
fi

set -- ${args}

dryrun=0

for a do
        case "${a}" in
 
        -n)
                dryrun=1
                shift
                ;;

        --)
                shift
                break
                ;;
        esac
done

if [ $# -ne 0 ]; then
        usage
fi

createdir() {
	local dir mode
	if [ $# -ne 2 ]; then
		echo "${prog}: createdir: bad args" 1>&2
		exit 1
	fi
	dir=$1
	mode=$2

	if [ ! -d ${dir} ]; then
		cmd="mkdir -p ${dir}"
		echo "+ ${cmd}"
		if [ ${dryrun} -eq 0 ]; then
			${cmd} || exit 1
		fi

		cmd="chown 0:0 ${dir}"
		echo "+ ${cmd}"
		if [ ${dryrun} -eq 0 ]; then
			${cmd} || exit 1
		fi

		cmd="chmod ${mode} ${dir}"
		echo "+ ${cmd}"
		if [ ${dryrun} -eq 0 ]; then
			${cmd} || exit 1
		fi

		cmd="ls -ld ${dir}"
		echo "+ ${cmd}"
		if [ ${dryrun} -eq 0 ]; then
			${cmd} || exit 1
		fi
	fi
}

# Define replacement shell function if running on linux
realpath /dev/null >/dev/null 2>&1
if [ $? -ne 0 ]; then
	realpath() {
		if [ ! -h $1 ]; then
			echo $1
			return
		fi
		dest="`readlink $1`"
		case "${dest}" in

		/*)
			echo ${dest}
			;;
		*)
			dir="`dirname $1`"
			echo "${dir}/${dest}"
		esac
	}
fi

. /etc/rc.conf

if [ -z "${captive_enable}" -o "x${captive_enable}" != "xYES" ] ; then
	echo "${prog}: captive_enable is not enabled in /etc/rc.conf" 1>&2
	exit 1
fi

# Files
files="/etc/localtime"
for file in /etc/termcap /usr/share/misc/termcap /usr/share/misc/termcap.db ; do
	if [ -r ${file} ]; then
		files="${files} ${file}"
	fi
done
files="${files} ${captive_files}"

# Bins
bins="/bin/csh ${captive_bins}"

# Libs
libs=""
for file in /libexec/ld-* /lib/ld-*; do
	if [ -r ${file} ]; then
		libs="${libs} ${file}"
	fi
done
libs="${libs} ${captive_libs}"

# Generate list of shared libraries
rm -f ${t1}
for file in ${bins}; do
	if [ ! -x ${file} ]; then
		continue
	fi
	ldd ${file} 2>&1 |
	    awk '/not a dynamic executable/ {
		    exit
	    }
	    $2 == "=>" {
		    print $3
	    }' >> ${t1}
done
sharedlibs="`sort -u ${t1}`"

# Resolve symlinks
for file in ${sharedlibs}; do
        realpath ${file} >> ${t1}
done
sharedlibs="`sort -u ${t1}`"
rm ${t1}

# Get rid of duplicates
files="`echo ${files} | fmt -1 | sort -u`"
bins="`echo ${bins} | fmt -1 | sort -u`"
libs="`echo ${libs} | fmt -1 | sort -u`"
sharedlibs="`echo ${sharedlibs} | fmt -1 | sort -u`"

# Create directories
createdir ~captive 755
createdir ~captive/dev 555
createdir ~captive/tmp 1777

createdir ~captive/var 755
createdir ~captive/var/log 755
createdir ~captive/var/spool 755
createdir ~captive/var/spool/lock 1777
createdir ~captive/var/tmp 1777
createdir ~captive/var/tmp/vi.recover 1777

# Update files, bins, libs and sharedlibs
rsync="rsync -aiHRO --delete $*"
if [ ${dryrun} -ne 0 ]; then
	rsync="${rsync} -n"
fi
${rsync} ${files} ${bins} ${libs} ${sharedlibs} ~captive > ${t1} 2>&1
if [ $? -ne 0 ]; then
	echo "+ ${rsync} [files ...] ~captive"
	cat ${t1}
	rm ${t1}
	exit 1
fi

# Only print if we updated files
if [ -s ${t1} ]; then
	echo "+ ${rsync} [files ...] ~captive"
	cat ${t1}
fi
rm ${t1}
